@@ -21,6 +21,3 @@ class InputMode(object):
 
     #: Vi-style forward search. Usually with a '/' or '?' prompt.
     VI_SEARCH = 'vi-forward-search'
-
-    #: Ctrl-N/Ctrl-P style navigation through completions.
-    COMPLETE = 'complete'
