@@ -3,6 +3,7 @@ Parser for VT100 input stream.
 """
 from __future__ import unicode_literals
 import six
+import re
 
 from .keys import Keys
 
@@ -10,6 +11,12 @@ __all__ = (
     'InputStream',
 )
 
+# Regex matching any CPR response
+_cpr_response_re = re.compile('^' + re.escape('\x1b[') + r'\d+;\d+R$')
+
+# Regex matching any valid prefix of a CPR response.
+_cpr_response_prefix_re = re.compile('^' + re.escape('\x1b[') + r'[\d;R]*$')
+
 
 class KeyPress(object):
     """
@@ -23,6 +30,9 @@ class KeyPress(object):
     def __repr__(self):
         return 'KeyPress(key=%r, data=%r)' % (self.key, self.data)
 
+    def __eq__(self, other):
+        return self.key == other.key and self.data == other.data
+
 
 class _Flush(object):
     """ Helper object to indicate flush operation to the parser. """
@@ -135,22 +145,23 @@ class InputStream(object):
         '\x1b[1;3B': (Keys.Escape, Keys.Down),
     }
 
-    def __init__(self, input_processor, stdout=None):
+    def __init__(self, input_processor):
         self._input_processor = input_processor
-        self._stdout = stdout
-
         self.reset()
 
-    def reset(self):
+    def reset(self, request=False):
         self._start_parser()
 
+    def prepare_terminal(self, stdout):
+        """
+        Prepare terminal.
+        """
         # Put the terminal in cursor mode. (Instead of application mode.)
-        if self._stdout:
-            self._stdout.write('\x1b[?1l')
-            self._stdout.flush()
+        stdout.write('\x1b[?1l')
+        stdout.flush()
 
-            # # Put the terminal in application mode.
-            # self._stdout.write('\x1b[?1h')
+        # # Put the terminal in application mode.
+        # self._stdout.write('\x1b[?1h')
 
     def _start_parser(self):
         """
@@ -163,12 +174,25 @@ class InputStream(object):
         """
         Return the keys that map to this prefix.
         """
+        # (hard coded) If we match a CPR response, return Keys.CPRResponse.
+        # (This one doesn't fit in the mappings, because it contains
+        # integer variables.)
+        if _cpr_response_re.match(prefix):
+            return [Keys.CPRResponse]
+
+        # Otherwise, use the mappings.
         return [v for k, v in self.mappings.items() if k == prefix]
 
     def _is_prefix_of_longer_match(self, prefix):
         """
         True when there is any key that start with this characters.
         """
+        # (hard coded) If this could be a prefix of a CPR response, return
+        # True.
+        if _cpr_response_prefix_re.match(prefix):
+            return True
+
+        # If this could be a prefix of anything else, also return True.
         return any(v for k, v in self.mappings.items() if k.startswith(prefix) and k != prefix)
 
     def _input_parser_generator(self):
