@@ -8,9 +8,37 @@ from .utils import create_handle_decorator
 
 
 def basic_bindings(registry, cli_ref):
+    cli = cli_ref()
     line = cli_ref().line
+    renderer = cli_ref().renderer
     handle = create_handle_decorator(registry, line)
 
+    @handle(Keys.ControlA)
+    @handle(Keys.ControlB)
+    @handle(Keys.ControlC)
+    @handle(Keys.ControlD)
+    @handle(Keys.ControlE)
+    @handle(Keys.ControlF)
+    @handle(Keys.ControlG)
+    @handle(Keys.ControlH)
+    @handle(Keys.ControlI)
+    @handle(Keys.ControlJ)
+    @handle(Keys.ControlK)
+    @handle(Keys.ControlL)
+    @handle(Keys.ControlM)
+    @handle(Keys.ControlN)
+    @handle(Keys.ControlO)
+    @handle(Keys.ControlP)
+    @handle(Keys.ControlQ)
+    @handle(Keys.ControlR)
+    @handle(Keys.ControlS)
+    @handle(Keys.ControlT)
+    @handle(Keys.ControlU)
+    @handle(Keys.ControlV)
+    @handle(Keys.ControlW)
+    @handle(Keys.ControlX)
+    @handle(Keys.ControlY)
+    @handle(Keys.ControlZ)
     @handle(Keys.F1)
     @handle(Keys.F2)
     @handle(Keys.F3)
@@ -35,6 +63,7 @@ def basic_bindings(registry, cli_ref):
     @handle(Keys.ControlBackslash)
     @handle(Keys.ControlSquareClose)
     @handle(Keys.ControlCircumflex)
+    @handle(Keys.ControlUnderscore)
     @handle(Keys.Backspace)
     @handle(Keys.Up)
     @handle(Keys.Down)
@@ -52,8 +81,11 @@ def basic_bindings(registry, cli_ref):
         """
         First, for any of these keys, Don't do anything by default. Also don't
         catch them in the 'Any' handler which will insert them as data.
+
+        If people want to insert these characters as a literal, they can always
+        do by doing a quoted insert. (ControlQ in emacs mode, ControlV in Vi
+        mode.)
         """
-        # We override the functionality below.
         pass
 
     @handle(Keys.Home, in_mode=InputMode.INSERT)
@@ -70,7 +102,7 @@ def basic_bindings(registry, cli_ref):
 
     @handle(Keys.ControlC)
     def _(event):
-        line.abort()
+        cli.set_abort()
 
     @handle(Keys.ControlD)
     def _(event):
@@ -78,14 +110,13 @@ def basic_bindings(registry, cli_ref):
         if line.text:
             line.delete()
         else:
-            line.exit()
+            cli.set_exit()
 
     @handle(Keys.ControlE)
     def _(event):
         line.cursor_position += line.document.get_end_of_line_position()
 
     @handle(Keys.ControlI, in_mode=InputMode.INSERT)
-    @handle(Keys.ControlI, in_mode=InputMode.COMPLETE)
     def _(event):
         r"""
         Ctrl-I is identical to "\t"
@@ -93,21 +124,38 @@ def basic_bindings(registry, cli_ref):
         Traditional tab-completion, where the first tab completes the common
         suffix and the second tab lists all the completions.
         """
-        # On the second tab-press:
-        if event.second_press:
-            line.list_completions()
+        def second_tab():
+            line.complete_next(start_at_first=False)
+
+        # On the second tab-press, or when already navigating through
+        # completions.
+        if event.second_press or (line.complete_state and line.complete_state.complete_index is not None):
+            second_tab()
         else:
-            not line.complete_common()
+            # On the first tab press, only complete the common parts of all completions.
+            has_common = line.complete_common()
+            if not has_common:
+                second_tab()
+
+    @handle(Keys.BackTab, in_mode=InputMode.INSERT)
+    def _(event):
+        """
+        Shift+Tab: go to previous completion.
+        """
+        line.complete_previous()
 
     @handle(Keys.ControlJ, in_mode=InputMode.INSERT)
     @handle(Keys.ControlM, in_mode=InputMode.INSERT)
-    @handle(Keys.ControlJ, in_mode=InputMode.COMPLETE)
-    @handle(Keys.ControlM, in_mode=InputMode.COMPLETE)
     def _(event):
         """
         Newline/Enter. (Or return input.)
         """
-        line.auto_enter()
+        if line.is_multiline:
+            line.newline()
+        else:
+            if line.validate():
+                cli_ref().line.add_to_history()
+                cli_ref().set_return_value(line.document)
 
     @handle(Keys.ControlK, in_mode=InputMode.INSERT)
     def _(event):
@@ -116,7 +164,7 @@ def basic_bindings(registry, cli_ref):
 
     @handle(Keys.ControlL)
     def _(event):
-        line.clear()
+        renderer.clear()
 
     @handle(Keys.ControlT, in_mode=InputMode.INSERT)
     def _(event):
@@ -141,18 +189,10 @@ def basic_bindings(registry, cli_ref):
             deleted = line.delete_before_cursor(count=-pos)
             line.set_clipboard(ClipboardData(deleted))
 
-    @handle(Keys.PageUp, in_mode=InputMode.COMPLETE)
-    def _(event):
-        line.complete_previous(5)
-
     @handle(Keys.PageUp, in_mode=InputMode.INSERT)
     def _(event):
         line.history_backward()
 
-    @handle(Keys.PageDown, in_mode=InputMode.COMPLETE)
-    def _(event):
-        line.complete_next(5)
-
     @handle(Keys.PageDown, in_mode=InputMode.INSERT)
     def _(event):
         line.history_forward()
@@ -183,25 +223,11 @@ def basic_bindings(registry, cli_ref):
     def _(event):
         line.cursor_down(count=event.arg)
 
-    @handle(Keys.Up, in_mode=InputMode.COMPLETE)
-    def _(event):
-        line.complete_previous()
-
-    @handle(Keys.Down, in_mode=InputMode.COMPLETE)
-    def _(event):
-        line.complete_next()
-
     @handle(Keys.ControlH, in_mode=InputMode.INSERT)
     @handle(Keys.Backspace, in_mode=InputMode.INSERT)
-    @handle(Keys.ControlH, in_mode=InputMode.COMPLETE)
-    @handle(Keys.Backspace, in_mode=InputMode.COMPLETE)
     def _(event):
         line.delete_before_cursor(count=event.arg)
 
-        # Quit autocomplete mode.
-        if event.input_processor.input_mode == InputMode.COMPLETE:
-            event.input_processor.pop_input_mode()
-
     @handle(Keys.Delete, in_mode=InputMode.INSERT)
     def _(event):
         line.delete(count=event.arg)
@@ -210,7 +236,6 @@ def basic_bindings(registry, cli_ref):
     def _(event):
         line.delete(count=event.arg)
 
-    @handle(Keys.Any, in_mode=InputMode.COMPLETE)
     @handle(Keys.Any, in_mode=InputMode.INSERT)
     def _(event):
         """
@@ -218,14 +243,14 @@ def basic_bindings(registry, cli_ref):
         """
         line.insert_text(event.data * event.arg)
 
-        # Quit autocomplete mode.
-        if event.input_processor.input_mode == InputMode.COMPLETE:
-            event.input_processor.pop_input_mode()
-
-    @handle(Keys.Escape, in_mode=InputMode.COMPLETE)
-    @handle(Keys.ControlC, in_mode=InputMode.COMPLETE)
+    @handle(Keys.CPRResponse)
     def _(event):
         """
-        Pressing escape or Ctrl-C in complete mode, goes back to default mode.
+        Handle incoming Cursor-Position-Request response.
         """
-        event.input_processor.pop_input_mode()
+        # The incoming data looks like u'\x1b[35;1R'
+        # Parse row/col information.
+        row, col = map(int, event.data[2:-1].split(';'))
+
+        # Report absolute cursor position to the renderer.
+        cli_ref().renderer.report_absolute_cursor_row(row)
