@@ -1,8 +1,8 @@
 from __future__ import unicode_literals
-from ..line import ClipboardData, ClipboardDataType
-from ..selection import SelectionType
 from ..enums import IncrementalSearchDirection, InputMode
 from ..keys import Keys
+from ..line import ClipboardData, ClipboardDataType
+from ..selection import SelectionType
 
 from .basic import basic_bindings
 from .utils import create_handle_decorator
@@ -115,31 +115,36 @@ def vi_bindings(registry, cli_ref):
         line.insert_text(event.data, overwrite=False)
 
     @handle(Keys.ControlN, in_mode=InputMode.INSERT)
-    @handle(Keys.ControlN, in_mode=InputMode.COMPLETE)
     def _(event):
         line.complete_next()
 
-        # Switch only to the 'complete' input mode if there really was a
-        # completion found.
-        if line.complete_state and event.input_processor.input_mode != InputMode.COMPLETE:
-            event.input_processor.push_input_mode(InputMode.COMPLETE)
-
     @handle(Keys.ControlN, in_mode=InputMode.VI_NAVIGATION)
     def _(event):
         """
-        CtrlN in navigation mode goes down.
+        Control-N: Next completion.
         """
         line.auto_down()
 
     @handle(Keys.ControlP, in_mode=InputMode.INSERT)
-    @handle(Keys.ControlP, in_mode=InputMode.COMPLETE)
     def _(event):
+        """
+        Control-P: To previous completion.
+        """
         line.complete_previous()
 
-        # Switch only to the 'complete' input mode if there really was a
-        # completion found.
-        if line.complete_state and event.input_processor.input_mode != InputMode.COMPLETE:
-            event.input_processor.push_input_mode(InputMode.COMPLETE)
+    @handle(Keys.ControlY, in_mode=InputMode.INSERT)
+    def _(event):
+        """
+        Accept current completion.
+        """
+        line.complete_state = None
+
+    @handle(Keys.ControlE, in_mode=InputMode.INSERT)
+    def _(event):
+        """
+        Cancel completion. Go back to originally typed text.
+        """
+        line.cancel_completion()
 
     @handle(Keys.ControlP, in_mode=InputMode.VI_NAVIGATION)
     def _(event):
@@ -154,7 +159,9 @@ def vi_bindings(registry, cli_ref):
         """
         In navigation mode, pressing enter will always return the input.
         """
-        line.return_input()
+        if line.validate():
+            line.add_to_history()
+            cli_ref().set_return_value(line.document)
 
     # ** In navigation mode **
 
@@ -886,9 +893,8 @@ def vi_bindings(registry, cli_ref):
         """
         Enter at the / or ? prompt.
         """
-        # Call return_input to put the current result on the history stack
-        # of search operations.
-        search_line.return_input(call_callback=False)
+        # Add query to history of searh line.
+        search_line.add_to_history()
         search_line.reset()
 
         # Go back to navigation mode.
@@ -961,3 +967,19 @@ def vi_bindings(registry, cli_ref):
 
     for k, f in vi_transform_functions:
         create_selection_transform_handler(k, f)
+
+    @handle(Keys.ControlX, Keys.ControlL, in_mode=InputMode.INSERT)
+    def _(event):
+        """
+        Pressing the ControlX - ControlL sequence in Vi mode does line
+        completion based on the other lines in the document and the history.
+        """
+        line.start_history_lines_completion()
+
+    @handle(Keys.ControlX, Keys.ControlF, in_mode=InputMode.INSERT)
+    def _(event):
+        """
+        Complete file names.
+        """
+        # TODO
+        pass
